package core.services;

import core.api.models.timemanager.history.ActivityInfo;
import core.api.models.timemanager.history.TaskgroupActivityInfo;
import core.api.models.timemanager.taskgroup.TaskgroupEntityInfo;
import core.entities.timemanager.AbstractSchedule;
import core.entities.timemanager.Taskgroup;
import core.repositories.timemanager.ScheduleRepository;
import core.repositories.timemanager.TaskgroupRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;

@Service
public class StatisticService {

    @Autowired private ScheduleRepository scheduleRepository;

    public HashMap<Taskgroup, HashMap<LocalDate, Integer>>  calcActivityByUser(Taskgroup taskgroup, String username, LocalDate startinDate, LocalDate endingDate) {
        HashMap<Taskgroup, HashMap<LocalDate, Integer>> taskgroupActivityInfos = new HashMap<>();
        List<AbstractSchedule> startedSchedules = scheduleRepository.getAllFinishedSchedulesByUser(username);
        for(AbstractSchedule schedule : startedSchedules) {
            if(schedule.getStartTime().toLocalDate().isAfter(endingDate) || schedule.getStopTime().toLocalDate().isBefore(startinDate)) {
                continue;
            }

            Taskgroup topTaskgroup;
            if(taskgroup == null) {
                topTaskgroup = findTopTaskgroup(schedule.getTask().getTaskgroup());
            } else {
                topTaskgroup = findTaskgroupOfLayer(taskgroup, schedule.getTask().getTaskgroup());
                if(topTaskgroup == null) {
                    continue;
                }
            }

            HashMap<LocalDate, Integer> taskgroupActivity;
            if(taskgroupActivityInfos.containsKey(topTaskgroup)) {
                taskgroupActivity = taskgroupActivityInfos.get(topTaskgroup);
            } else {
                taskgroupActivity = new HashMap<>();
                taskgroupActivityInfos.put(topTaskgroup, taskgroupActivity);
            }

            if(schedule.getStartTime().toLocalDate().isEqual(schedule.getStopTime().toLocalDate())) {
                insertActivity(taskgroupActivity, schedule.getStartTime().toLocalDate(), schedule.getActiveTime());
            } else {
                //Starting Date
                LocalDateTime startingDayEnd = schedule.getStartTime().toLocalDate().atTime(LocalTime.MAX);
                Duration startingActivity = Duration.between(schedule.getStartTime(), startingDayEnd);
                insertActivity(taskgroupActivity, schedule.getStartTime().toLocalDate(), (int) startingActivity.toMinutes());

                //Ending Date
                LocalDateTime endingDayStart = schedule.getStopTime().toLocalDate().atStartOfDay();
                Duration endingActivity = Duration.between(endingDayStart, schedule.getStopTime());
                insertActivity(taskgroupActivity, schedule.getStopTime().toLocalDate(), (int) endingActivity.toMinutes());
            }
        }

        return taskgroupActivityInfos;
    }

    public static List<TaskgroupActivityInfo> convertInternActivityInfo(HashMap<Taskgroup, HashMap<LocalDate, Integer>> taskgroupActivity) {
        List<TaskgroupActivityInfo> taskgroupActivityInfos = new ArrayList<>();
        for(Map.Entry<Taskgroup, HashMap<LocalDate, Integer>> entry: taskgroupActivity.entrySet()) {
            List<ActivityInfo> activityInfos = new ArrayList<>();
            for(Map.Entry<LocalDate, Integer> dateActivity : entry.getValue().entrySet()) {
                activityInfos.add(new ActivityInfo(dateActivity.getKey(), dateActivity.getValue()));
            }
            taskgroupActivityInfos.add(new TaskgroupActivityInfo(new TaskgroupEntityInfo(entry.getKey()), activityInfos));
        }
        return taskgroupActivityInfos;
    }

    private static void insertActivity(HashMap<LocalDate, Integer> activityInfo, LocalDate date, int deltaActivity) {
        if(activityInfo.containsKey(date)) {
            int activity = activityInfo.get(date);
            activity += deltaActivity;
            activityInfo.put(date, activity);
        } else {
            activityInfo.put(date, deltaActivity);
        }
    }

    private Taskgroup findTopTaskgroup(Taskgroup taskgroup) {
        Taskgroup currentTaskgroup = taskgroup;
        while(currentTaskgroup.getParent() != null) {
            currentTaskgroup = currentTaskgroup.getParent();
        }
        return currentTaskgroup;
    }

    private Taskgroup findTaskgroupOfLayer(Taskgroup targetLayer, Taskgroup childTaskgroup) {
        if(targetLayer.getTaskgroupID() == childTaskgroup.getTaskgroupID()) {
            return childTaskgroup;
        }

        Taskgroup currentTaskgroup = childTaskgroup;
        while (currentTaskgroup.getParent() != null && currentTaskgroup.getParent().getTaskgroupID() != targetLayer.getTaskgroupID()) {
            currentTaskgroup = currentTaskgroup.getParent();
        }

        if(currentTaskgroup.getParent() == null) {
            return null;
        }
        return currentTaskgroup;
    }
}
