import { Injectable } from '@angular/core';
import {TaskEntityInfo} from "../../api";
import * as moment from "moment";

export enum TaskStatus {
  OK,
  WARNING,
  ATTENTION,
  DEAD
}
@Injectable({
  providedIn: 'root'
})
export class TaskStatusService {

  constructor() { }

  computeTaskStatus(task: TaskEntityInfo): TaskStatus {
    if(!task.finished &&  task.deadline != null && moment(task.deadline, 'YYYY-MM-DDTHH:mm:ss.SSSZ').isBefore(moment())) {
      return TaskStatus.DEAD;
    } else if(!task.finished && task.deadline != null && !task.hasPlannedSchedules) {
      return TaskStatus.ATTENTION
    } else if(!task.finished && !task.hasPlannedSchedules) {
      return TaskStatus.WARNING
    } else {
      return TaskStatus.OK;
    }
  }

  convertTaskStatusToColor(taskStatus: TaskStatus) {
    switch (taskStatus) {
      case TaskStatus.OK: return "🟢";
      case TaskStatus.WARNING: return "🟠";
      case TaskStatus.ATTENTION: return "🔴";
      case TaskStatus.DEAD: return "⚫";
    }
  }
}
